; PUTKBD2 - LEM to stuff keyboard buffer with up to 15 ascii characters
;	AND/OR extended keycodes.
;
; This LEM is intended as a direct replacement for Mike Hanson's PUTKBD LEM.
;
; PUTKBD proved to be very useful to me and I used it regularly in my apps.
; I eventually needed to stuff the Esc_Key as well as function keys to Clarion,
; and is where his LEM came up a little short. I tried simply PUTKBD('<27>'),
; for the escape key, but all that did was cause Clarion to display the escape
; character in my entry field.
;
; The problem is that Clarion checks for the COMPLETE scan code on ALERTed keys,
; which would be 011Bh for the Escape key.
;
; PUTKBD2 allows passing of extended scan codes as well as ASCII strings.
;
; If you do not use any extended scan codes, it will function exactly as
; PUTKBD and save you 100 bytes memory.
;
; I have included two versions of the .BIN, PUTKBD2.BIN, and PUTKBD.BIN, both
; function identically, PUTKBD.BIN is included so that if you currently use
; Mike Hanson's version you will not have to make any changes to your code.
;
; SYNTAX: PUTKBD2(String)
;   Example: PUTKBD2('DIR<13>'), could be used to pass DIR command to DOS
;
;   If you pass a straight ASCII string, that is exactly what the buffer
;   will return to your program, just as if you had typed it.
;
;   In order to pass an extended keycode you must use an 'escape sequence'.
;   I chose to use a CHR(0) followed by the full 2 byte extended BIOS scancode
;   in HI Byte/LO Byte order.  The HI Byte is the Extended portion of the
;   Scan Code, and the Low Byte is the ASCII portion. Consult an MS-DOS
;   reference for the appropriate scan codes.
;
;   Example:
;   The following would pass an ascii string('Jerry') followed by the F10_Key:
;
;		    First 0 says to treat the following
;		    		    two bytes as an extended code
;		    
;		    	Extended portion of Scan Code
;		    	
;		    	  ASCII portion of code
;   PUTKBD2('Jerry< 0, 68, 0>')
;
;
;   The following would pass an ascii string('Jerry') followed by the Esc_Key
;   followed by another string(' Murdock').
;		    First 0 says to treat the following
;		    		    two bytes as an extended code
;		    
;		      Extended portion of Scan Code
;		      
;		         ASCII portion of Scan Code(decimal 27)
;   PUTKBD2('Jerry< 0,01h,1Bh> Murdock')
;
;
;   Any size string may be passed, but anything past 15 scan codes will be
;   truncated. For instance, in the second example above, the actual string
;   passed to the LEM would be 16 bytes long, but would translate into only
;   14 scan codes.
;
; MAP structure:
;   You must include the following in your map structure:
;   MODULE('PUTKBD2'),BINARY
;	  PROC(PUTKBD2)
;   .
;
;
; Written by: Jerry Murdock
;	      3118 Greenwood Trail
;	      Marietta, GA 30067
;
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
; I believe PUTKBD2 will function as described, it has never caused any
; problems on my system.  However, I make no guarantees that it is bug free and
; take no responsibility for any problems that may arise from its use.
; Use at your own risk.

BIN  SEGMENT BYTE
     ASSUME CS:BIN,DS:BIN


DB	'BIO'				    ;SIGNATURE
	 DD 0				    ;RESERVED
	 DW BINEND
	 DB 1				    ;ONE FUNCTION IN MODULE

	 DB 'PUTKBD2',0,0,0,0,0,0	    ;FUNCTION NAME
	 DW PUTKBD			    ;FUNCTION OFFSET
	 DB 0				    ;SPECIFY A PROCEDURE
	 DB 1				    ;WITH ONE PARAMETER

	 DB 0				    ;FIRST PARAMETER IS A STRING
STUFSTR	 DD 0				    ;ADDRESS OF STRING
STR_LEN	 DW 0				    ;LENGTH OF STRING

;0040:001Ah = Keyboard buffer head pointer
;0040:001Ch = Keyboard buffer tail pointer
;0040:001Eh = Buffer starts at 0040:001Eh



PUTKBD	 PROC	FAR

	 MOV	CX,STR_LEN		   ;SET CX TO LENGTH OF STRING

	 MOV	AX,0040h
	 MOV	ES,AX			   ;SET ES TO KEYBOARD BUFFER SEGMENT
	 MOV	DI,001Ah
	 MOV	WORD PTR ES:[DI],001Eh	   ;SET POINTER TO BEGINING OF BUFFER


	 MOV	DI,001Ch		   ;SET DI TO TAIL POINTER ADDRESS
	 MOV	WORD PTR ES:[DI],001Eh	   ;SET TAIL = HEAD, CLEARING BUFFER

	 MOV	DI,001Eh		   ;SET DI TO BEG OF COPY

	 LDS	SI,STUFSTR		   ;LOAD DS:SI WITH ADDRESS OF STRING



BEG_OF_LOOP:

	 JCXZ	END_OF_LOOP		;IF CX=0, WE'RE DONE W/STRING, EXIT LOOP

	 CMP	DI,003Bh		   ;COMPARE DI WITH END OF BUFFER
	 JG END_OF_LOOP			   ;IF WE'VE FILLED BUFFER, EXIT LOOP

	 LODSB				   ;LOAD FIRST BYTE OF STRING
	 DEC	CX			   ;INCREMENT STR_LENGTH COUNTER

	 CMP	AL,0			   ;IS THIS A CHR(0)
	 JNE	NOT_EXTENDED		   ;NO - PROCESS AS NOT_EXTENDED

EXTENDED_CODE:
	 LODSB				   ;LOAD EXTENDED PART OF SCAN CODE
	 DEC	CX			   ;DECREMENT STRING COUNTER
	 MOV	BL,AL			   ;SAVE EXT. SCAN CODE TEMPORARILY
	 LODSB				   ;LOAD ASCII PORTION
	 DEC	CX			   ;DECREMENT STRING COUNTER
	 STOSB				   ;STUFF ASCII PORTION IN BUFFER
	 MOV	AL,BL			   ;RESTORE EXTENDED PORTION
	 STOSB				   ;STUFF EXT. PORTION IN BUFFER

	 JMP	BEG_OF_LOOP		   ;LOOP


NOT_EXTENDED:
	 STOSB				   ;STORE ASCII PORTION OF SCAN CODE
	 MOV	BYTE PTR ES:[DI],0	   ;SET EXT. PORTION TO 0
	 INC	DI			   ;INCREMENT DI PAST EXTENDED PORTION

	 JMP	BEG_OF_LOOP		   ;LOOP

END_OF_LOOP:
	 MOV	CX,DI			   ;SAVE DI (DI = POINTER TO TAIL)
	 MOV	DI,001Ch		   ;LOAD ADDRESS OF TAIL POINTER
	 MOV	ES:[DI],CX		   ;WITH OFFSET TO END OF STRING

RET_TO_CLA:
	 RET				   ;RETURN TO CLARION

PUTKBD	 ENDP				   ;END OF PROCEDURE
BINEND	 DB 0				   ;END OF LEM
BIN  ENDS
     END
